#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

range = getattr(__builtins__, 'xrange', range)
# end of py2 compatability boilerplate

import os

import pytest

import numpy as np

from matrixprofile.algorithms.mstomp import mstomp


def test_mstomp_window_size_less_than_4():
    ts = np.array([[1, 2, 3, 4, 5, 6, 7, 8], [8, 7, 6, 5, 4, 3, 2, 1]])
    w = 2

    with pytest.raises(ValueError) as excinfo:
        mstomp(ts, w)
        assert 'window size must be at least 4.' in str(excinfo.value)


def test_mstomp_time_series_too_small():
    ts = np.array([[1, 2, 3, 4, 5, 6, 7, 8], [8, 7, 6, 5, 4, 3, 2, 1]])
    w = 8

    with pytest.raises(ValueError) as excinfo:
        mstomp(ts, w)
        assert 'Time series is too short' in str(excinfo.value)


def test_mstomp_single_dimension():
    ts = np.array([0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0])
    w = 4
    desired_mp = np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0]])
    desired_pi = np.array([[4, 5, 6, 7, 0, 1, 2, 3, 0]])

    desired_lmp = np.array([[np.inf, np.inf, np.inf, 2.82842712, 0, 0, 0, 0, 0]])
    desired_lpi = np.array([[0, 0, 0, 0, 0, 1, 2, 3, 0]])

    desired_rmp = np.array([[0, 0, 0, 0, 0, 2.82842712, np.inf, np.inf, np.inf]])
    desired_rpi = np.array([[4, 5, 6, 7, 8, 8, 0, 0, 0]])

    profile = mstomp(ts, w, n_jobs=1)
    np.testing.assert_almost_equal(profile['mp'], desired_mp)
    np.testing.assert_almost_equal(profile['pi'], desired_pi)

    np.testing.assert_almost_equal(profile['lmp'], desired_lmp)
    np.testing.assert_almost_equal(profile['lpi'], desired_lpi)

    np.testing.assert_almost_equal(profile['rmp'], desired_rmp)
    np.testing.assert_almost_equal(profile['rpi'], desired_rpi)


def test_mstomp_multi_dimension():
    ts = np.array([[0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0], [1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1]])
    w = 4
    desired_mp = np.array(
        [[0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 9.19401687e-01, 9.19401687e-01, 2.98023224e-08, 0, 9.19401687e-01, 9.19401687e-01, 9.19401687e-01]])
    desired_pi = np.array([[4, 5, 6, 7, 0, 1, 2, 3, 0], [4, 5, 6, 7, 0, 1, 2, 3, 0]])
    desired_pd = [
        np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0]]),
        np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 1, 1, 1, 1, 1, 1, 1, 1]])
    ]

    profile = mstomp(ts, w, return_dimension=True, n_jobs=1)
    np.testing.assert_almost_equal(profile['mp'], desired_mp)
    np.testing.assert_almost_equal(profile['pi'], desired_pi)
    for i in range(len(ts)):
        np.testing.assert_almost_equal(profile['pd'][i], desired_pd[i])


def test_mstomp_single_dimension_multi_threaded():
    ts = np.array([0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0])
    w = 4
    desired_mp = np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0]])
    desired_pi = np.array([[4, 5, 6, 7, 0, 1, 2, 3, 0]])

    desired_lmp = np.array([[np.inf, np.inf, np.inf, 2.82842712, 0, 0, 0, 0, 0]])
    desired_lpi = np.array([[0, 0, 0, 0, 0, 1, 2, 3, 0]])

    desired_rmp = np.array([[0, 0, 0, 0, 0, 2.82842712, np.inf, np.inf, np.inf]])
    desired_rpi = np.array([[4, 5, 6, 7, 8, 8, 0, 0, 0]])

    profile = mstomp(ts, w, n_jobs=-1)
    np.testing.assert_almost_equal(profile['mp'], desired_mp)
    np.testing.assert_almost_equal(profile['pi'], desired_pi)

    np.testing.assert_almost_equal(profile['lmp'], desired_lmp)
    np.testing.assert_almost_equal(profile['lpi'], desired_lpi)

    np.testing.assert_almost_equal(profile['rmp'], desired_rmp)
    np.testing.assert_almost_equal(profile['rpi'], desired_rpi)


def test_mstomp_multi_dimension_multi_threaded():
    ts = np.array([[0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0], [1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1]])
    w = 4
    desired_mp = np.array(
        [[0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 9.19401687e-01, 9.19401687e-01, 2.98023224e-08, 0, 9.19401687e-01, 9.19401687e-01, 9.19401687e-01]])
    desired_pi = np.array([[4, 5, 6, 7, 0, 1, 2, 3, 0], [4, 5, 6, 7, 0, 1, 2, 3, 0]])
    desired_pd = [
        np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0]]),
        np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 1, 1, 1, 1, 1, 1, 1, 1]])
    ]

    profile = mstomp(ts, w, return_dimension=True, n_jobs=-1)
    np.testing.assert_almost_equal(profile['mp'], desired_mp)
    np.testing.assert_almost_equal(profile['pi'], desired_pi)
    for i in range(len(ts)):
        np.testing.assert_almost_equal(profile['pd'][i], desired_pd[i])


import setuptools

from setuptools import dist
dist.Distribution().fetch_build_eggs(['cython>=0.x', 'numpy>=1.16.2', 'wheel'])

from distutils.extension import Extension

from Cython.Build import cythonize
import numpy

import os, sys
from glob import glob

DIR_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(DIR_PATH)

import version

SOURCE_URL = 'https://github.com/aiacovella/matrixprofile.git'
README = os.path.join(DIR_PATH, 'README.rst')

# manual list of files to be compiled
extensions = []
extensions.append(Extension(
    'matrixprofile.algorithms.cympx',
    ['matrixprofile/algorithms/cympx.pyx'],
    extra_compile_args = ["-O2", "-fopenmp" ],
    extra_link_args = ['-fopenmp'],
    include_dirs=[numpy.get_include()],
))

extensions.append(Extension(
    'matrixprofile.cycore',
    ['matrixprofile/cycore.pyx'],
    extra_compile_args = ["-O2",],
    include_dirs=[numpy.get_include()],
))

matplot = 'matplotlib>=3.0.3'
scipy = 'scipy>=1.3.2,<2.0.0'
if sys.version_info.major == 3:
    with open(README, 'r', encoding='utf-8') as fh:
        long_description = fh.read()
elif sys.version_info.major == 2:
    matplot = 'matplotlib'
    scipy = 'scipy<2.0.0'
    with open(README, 'r') as fh:
        long_description = fh.read()

# copy version file over
with open(os.path.join(DIR_PATH, 'version.py')) as fh:
    with open(os.path.join(DIR_PATH, 'matrixprofile', 'version.py'), 'w') as out:
        out.write(fh.read())

setuptools.setup(
    name="peritus-matrixprofile",
    version=version.__version__,
    author="aiacovella",
    author_email="java.peritus@gmail.com",
    description="Fork of An open source time series data mining library based on Matrix Profile algorithms.",
    url=SOURCE_URL,
    project_urls={
        'Matrix Profile Foundation': 'https://matrixprofile.org',
        'Source Code': SOURCE_URL,
    },
    include_package_data=True,
    packages = setuptools.find_packages(),
    setup_requires=['cython>=0.x', 'wheel'],
    install_requires=['numpy>=1.16.2', matplot, 'protobuf>=3.11.2,<4.0.0', scipy],
    ext_modules=cythonize(extensions),
    include_dirs=[numpy.get_include()],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Topic :: Software Development",
        "Topic :: Scientific/Engineering",
    ],
    keywords="matrix profile time series discord motif analysis data science anomaly pattern",
)
